skhubness.neighbors.NearestNeighbors — scikit-hubness 0.21a1 documentation

# skhubness.neighbors.NearestNeighbors

class `skhubness.neighbors.` `NearestNeighbors`(n_neighbors=5, radius=1.0, algorithm: str = 'auto', algorithm_params: dict = None, hubness: str = None, hubness_params: dict = None, leaf_size=30, metric='minkowski', p=2, metric_params=None, n_jobs=None, **kwargs)

  Unsupervised learner for implementing neighbor searches.

  Read more in the scikit-learn User Guide.

  Parameters
    n_neighborsint, optional (default = 5)

      Number of neighbors to use by default for `kneighbors()` queries.

    radiusfloat, optional (default = 1.0)

      Range of parameter space to use by default for `radius_neighbors()` queries.

    algorithm{‘auto’, ‘hnsw’, ‘lsh’, ‘ball_tree’, ‘kd_tree’, ‘brute’}, optional

      Algorithm used to compute the nearest neighbors:

      - ‘hnsw’ will use `HNSW`

      - ‘lsh’ will use `LSH`

      - ‘ball_tree’ will use `BallTree`

      - ‘kd_tree’ will use `KDTree`

      - ‘brute’ will use a brute-force search.

      - ‘auto’ will attempt to decide the most appropriate algorithm based on the values passed to `fit()` method.

      Note: fitting on sparse input will override the setting of this parameter, using brute force.

    algorithm_paramsdict, optional

      Override default parameters of the NN algorithm. For example, with algorithm=’lsh’ and algorithm_params={n_candidates: 100} one hundred approximate neighbors are retrieved with LSH. If parameter hubness is set, the candidate neighbors are further reordered with hubness reduction. Finally, n_neighbors objects are used from the (optionally reordered) candidates.

    # TODO add all supported hubness reduction methods
    hubness{‘mutual_proximity’, ‘local_scaling’, ‘dis_sim_local’, None}, optional

      Hubness reduction algorithm - ‘mutual_proximity’ or ‘mp’ will use `MutualProximity' - 'local_scaling' or 'ls' will use :class:`LocalScaling` - ‘dis_sim_local’ or ‘dsl’ will use `DisSimLocal` If None, no hubness reduction will be performed (=vanilla kNN).

    hubness_params: dict, optional

      Override default parameters of the selected hubness reduction algorithm. For example, with hubness=’mp’ and hubness_params={‘method’: ‘normal’} a mutual proximity variant is used, which models distance distributions with independent Gaussians.

    leaf_sizeint, optional (default = 30)

      Leaf size passed to BallTree or KDTree. This can affect the speed of the construction and query, as well as the memory required to store the tree. The optimal value depends on the nature of the problem.

    metricstring or callable, default ‘minkowski’

      metric to use for distance computation. Any metric from scikit-learn or scipy.spatial.distance can be used.

      If metric is a callable function, it is called on each pair of instances (rows) and the resulting value recorded. The callable should take two arrays as input and return one value indicating the distance between them. This works for Scipy’s metrics, but is less efficient than passing the metric name as a string.

      Distance matrices are not supported.

      Valid values for metric are:

      - from scikit-learn: [‘cityblock’, ‘cosine’, ‘euclidean’, ‘l1’, ‘l2’, ‘manhattan’]

      - from scipy.spatial.distance: [‘braycurtis’, ‘canberra’, ‘chebyshev’, ‘correlation’, ‘dice’, ‘hamming’, ‘jaccard’, ‘kulsinski’, ‘mahalanobis’, ‘minkowski’, ‘rogerstanimoto’, ‘russellrao’, ‘seuclidean’, ‘sokalmichener’, ‘sokalsneath’, ‘sqeuclidean’, ‘yule’]

      See the documentation for scipy.spatial.distance for details on these metrics.

    pinteger, optional (default = 2)

      Parameter for the Minkowski metric from sklearn.metrics.pairwise.pairwise_distances. When p = 1, this is equivalent to using manhattan_distance (l1), and euclidean_distance (l2) for p = 2. For arbitrary p, minkowski_distance (l_p) is used.

    metric_paramsdict, optional (default = None)

      Additional keyword arguments for the metric function.

    n_jobsint or None, optional (default=None)

      The number of parallel jobs to run for neighbors search. `None` means 1 unless in a `joblib.parallel_backend` context. `-1` means using all processors. See Glossary for more details.

  See also

  `KNeighborsClassifier`
  `RadiusNeighborsClassifier`
  `KNeighborsRegressor`
  `RadiusNeighborsRegressor`
  `BallTree`

  Notes

  See Nearest Neighbors in the scikit-learn online documentation for a discussion of the choice of `algorithm` and `leaf_size`.

  https://en.wikipedia.org/wiki/K-nearest_neighbor_algorithm

  Examples

      >>> import numpy as np
      >>> from skhubness.neighbors import NearestNeighbors
      >>> samples = [[0, 0, 2], [1, 0, 0], [0, 0, 1]]

      >>> neigh = NearestNeighbors(2, 0.4)
      >>> neigh.fit(samples)  #doctest: +ELLIPSIS
      NearestNeighbors(...)

      >>> neigh.kneighbors([[0, 0, 1.3]], 2, return_distance=False)
      ... #doctest: +ELLIPSIS
      array([[2, 0]]...)

      >>> nbrs = neigh.radius_neighbors([[0, 0, 1.3]], 0.4, return_distance=False)
      >>> np.asarray(nbrs[0][0])
      array(2)

  `__init__`(self, n_neighbors=5, radius=1.0, algorithm: str = 'auto', algorithm_params: dict = None, hubness: str = None, hubness_params: dict = None, leaf_size=30, metric='minkowski', p=2, metric_params=None, n_jobs=None, **kwargs)

    Initialize self. See help(type(self)) for accurate signature.

  Methods

  `__init__`(self[, n_neighbors, radius, …]) | Initialize self.
  `fit`(self, X[, y]) | Fit the model using X as training data
  `get_params`(self[, deep]) | Get parameters for this estimator.
  `kcandidates`(self[, X, n_neighbors, …]) | Finds the K-neighbors of a point.
  `kneighbors`(self[, X, n_neighbors, …]) | TODO
  `kneighbors_graph`(self[, X, n_neighbors, mode]) | Computes the (weighted) graph of k-Neighbors for points in X
  `radius_neighbors`(self[, X, radius, …]) | Finds the neighbors within a given radius of a point or points.
  `radius_neighbors_graph`(self[, X, radius, mode]) | Computes the (weighted) graph of Neighbors for points in X
  `set_params`(self, \*\*params) | Set the parameters of this estimator.

  `fit`(self, X, y=None)

    Fit the model using X as training data

    Parameters
      X{array-like, sparse matrix, BallTree, KDTree}

        Training data. If array or matrix, shape [n_samples, n_features], or [n_samples, n_samples] if metric=’precomputed’.

  `get_params`(self, deep=True)

    Get parameters for this estimator.

    Parameters
      deepboolean, optional

        If True, will return the parameters for this estimator and contained subobjects that are estimators.

    Returns
      paramsmapping of string to any

        Parameter names mapped to their values.

  `kcandidates`(self, X=None, n_neighbors=None, return_distance=True) → numpy.ndarray

    Finds the K-neighbors of a point. Returns indices of and distances to the neighbors of each point.

    Parameters
      Xarray-like, shape (n_query, n_features), or (n_query, n_indexed) if metric == ‘precomputed’

        The query point or points. If not provided, neighbors of each indexed point are returned. In this case, the query point is not considered its own neighbor.

      n_neighborsint

        Number of neighbors to get (default is the value passed to the constructor).

      return_distanceboolean, optional. Defaults to True.

        If False, distances will not be returned

    Returns
      distarray

        Array representing the lengths to points, only present if return_distance=True

      indarray

        Indices of the nearest points in the population matrix.

    Examples

    In the following example, we construct a NeighborsClassifier class from an array representing our data set and ask who’s the closest point to [1,1,1]

        >>> samples = [[0., 0., 0.], [0., .5, 0.], [1., 1., .5]]
        >>> from skhubness.neighbors import NearestNeighbors
        >>> neigh = NearestNeighbors(n_neighbors=1)
        >>> neigh.fit(samples) # doctest: +ELLIPSIS
        NearestNeighbors(algorithm='auto', leaf_size=30, ...)
        >>> print(neigh.kneighbors([[1., 1., 1.]])) # doctest: +ELLIPSIS
        (array([[0.5]]), array([[2]]))

    As you can see, it returns [[0.5]], and [[2]], which means that the element is at distance 0.5 and is the third element of samples (indexes start at 0). You can also query for multiple points:

        >>> X = [[0., 1., 0.], [1., 0., 1.]]
        >>> neigh.kneighbors(X, return_distance=False) # doctest: +ELLIPSIS
        array([[1],
               [2]]...)

  `kneighbors`(self, X=None, n_neighbors=None, return_distance=True)

    TODO

  `kneighbors_graph`(self, X=None, n_neighbors=None, mode='connectivity')

    Computes the (weighted) graph of k-Neighbors for points in X

    Parameters
      Xarray-like, shape (n_query, n_features), or (n_query, n_indexed) if metric == ‘precomputed’

        The query point or points. If not provided, neighbors of each indexed point are returned. In this case, the query point is not considered its own neighbor.

      n_neighborsint

        Number of neighbors for each sample. (default is value passed to the constructor).

      mode{‘connectivity’, ‘distance’}, optional

        Type of returned matrix: ‘connectivity’ will return the connectivity matrix with ones and zeros, in ‘distance’ the edges are Euclidean distance between points.

    Returns
      Asparse matrix in CSR format, shape = [n_samples, n_samples_fit]

        n_samples_fit is the number of samples in the fitted data A[i, j] is assigned the weight of edge that connects i to j.

    See also

    `NearestNeighbors.radius_neighbors_graph`

    Examples

        >>> X = [[0], [3], [1]]
        >>> from sklearn.neighbors import NearestNeighbors
        >>> neigh = NearestNeighbors(n_neighbors=2)
        >>> neigh.fit(X) # doctest: +ELLIPSIS
        NearestNeighbors(algorithm='auto', leaf_size=30, ...)
        >>> A = neigh.kneighbors_graph(X)
        >>> A.toarray()
        array([[1., 0., 1.],
               [0., 1., 1.],
               [1., 0., 1.]])

  `radius_neighbors`(self, X=None, radius=None, return_distance=True)

    Finds the neighbors within a given radius of a point or points.

    Return the indices and distances of each point from the dataset lying in a ball with size `radius` around the points of the query array. Points lying on the boundary are included in the results.

    The result points are not necessarily sorted by distance to their query point.

    Parameters
      Xarray-like, (n_samples, n_features), optional

        The query point or points. If not provided, neighbors of each indexed point are returned. In this case, the query point is not considered its own neighbor.

      radiusfloat

        Limiting distance of neighbors to return. (default is the value passed to the constructor).

      return_distanceboolean, optional. Defaults to True.

        If False, distances will not be returned

    Returns
      distarray, shape (n_samples,) of arrays

        Array representing the distances to each point, only present if return_distance=True. The distance values are computed according to the `metric` constructor parameter.

      indarray, shape (n_samples,) of arrays

        An array of arrays of indices of the approximate nearest points from the population matrix that lie within a ball of size `radius` around the query points.

    Notes

    Because the number of neighbors of each point is not necessarily equal, the results for multiple query points cannot be fit in a standard data array. For efficiency, radius_neighbors returns arrays of objects, where each object is a 1D array of indices or distances.

    Examples

    In the following example, we construct a NeighborsClassifier class from an array representing our data set and ask who’s the closest point to [1, 1, 1]:

        >>> import numpy as np
        >>> samples = [[0., 0., 0.], [0., .5, 0.], [1., 1., .5]]
        >>> from skhubness.neighbors import NearestNeighbors
        >>> neigh = NearestNeighbors(radius=1.6)
        >>> neigh.fit(samples) # doctest: +ELLIPSIS
        NearestNeighbors(algorithm='auto', leaf_size=30, ...)
        >>> rng = neigh.radius_neighbors([[1., 1., 1.]])
        >>> print(np.asarray(rng[0][0])) # doctest: +ELLIPSIS
        [1.5 0.5]
        >>> print(np.asarray(rng[1][0])) # doctest: +ELLIPSIS
        [1 2]

    The first array returned contains the distances to all points which are closer than 1.6, while the second array returned contains their indices. In general, multiple points can be queried at the same time.

  `radius_neighbors_graph`(self, X=None, radius=None, mode='connectivity')

    Computes the (weighted) graph of Neighbors for points in X

    Neighborhoods are restricted the points at a distance lower than radius.

    Parameters
      Xarray-like, shape = [n_samples, n_features], optional

        The query point or points. If not provided, neighbors of each indexed point are returned. In this case, the query point is not considered its own neighbor.

      radiusfloat

        Radius of neighborhoods. (default is the value passed to the constructor).

      mode{‘connectivity’, ‘distance’}, optional

        Type of returned matrix: ‘connectivity’ will return the connectivity matrix with ones and zeros, in ‘distance’ the edges are Euclidean distance between points.

    Returns
      Asparse matrix in CSR format, shape = [n_samples, n_samples]

        A[i, j] is assigned the weight of edge that connects i to j.

    See also

    `kneighbors_graph`

    Examples

        >>> X = [[0], [3], [1]]
        >>> from sklearn.neighbors import NearestNeighbors
        >>> neigh = NearestNeighbors(radius=1.5)
        >>> neigh.fit(X) # doctest: +ELLIPSIS
        NearestNeighbors(algorithm='auto', leaf_size=30, ...)
        >>> A = neigh.radius_neighbors_graph(X)
        >>> A.toarray()
        array([[1., 0., 1.],
               [0., 1., 0.],
               [1., 0., 1.]])

  `set_params`(self, **params)

    Set the parameters of this estimator.

    The method works on simple estimators as well as on nested objects (such as pipelines). The latter have parameters of the form `<component>__<parameter>` so that it’s possible to update each component of a nested object.

    Returns
      self